﻿using System.Reflection;
using System.Windows;
using Autofac;
using Byond.AudioMidi;
using Byond.Standalone.MidiDotNetStuff;
using Byond.Standalone.PortAudioStuff;
using Byond.Standalone.ViewModels;
using Byond.Standalone.Views;

namespace Byond.Standalone
{
	public partial class App
	{
		private readonly FrameworkBootstrapper _frameworkBootstrapper;

		public App()
		{
			_frameworkBootstrapper = new FrameworkBootstrapper(
				cb =>
					{
						cb.RegisterType<PortAudioEngine>().AsSelf().As<IAudio>().As<IAudioInfo>().SingleInstance();
						cb.RegisterType<DotNetMidi>().AsSelf().As<IMidi>().SingleInstance();

						cb.RegisterType<MainWindow>().FindConstructorsWith(BindingFlags.NonPublic).SingleInstance();

						cb.RegisterType<HostControlsViewModel>();
						cb.RegisterType<HostControlsView>();
						
						cb.RegisterType<SettingsViewModel>();
						cb.RegisterType<SettingsView>();
						cb.RegisterType<AudioSettingsViewModel>();
						cb.RegisterType<AudioSettingsView>();
						cb.RegisterType<MidiSettingsViewModel>();
						cb.RegisterType<MidiSettingsView>();
					},
				(menuItemRegistrar, viewFactory) =>
					menuItemRegistrar.RegisterItem(() =>
						(UIElement) viewFactory.GetViewForModel<HostControlsViewModel>()));

			MainWindow = _frameworkBootstrapper.Container.Resolve<MainWindow>();
		}

		private void App_OnStartup(object sender, StartupEventArgs e)
		{
			MainWindow.Show();
		}

		protected override void OnExit(ExitEventArgs e)
		{
			base.OnExit(e);
			_frameworkBootstrapper.Dispose();
		}
	}
}
