﻿// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

using Byond.Standalone.PortAudioStuff;
using PortAudioSharp;

namespace Byond.Standalone.ViewModels
{
	class AudioDeviceViewModel
	{
		public static readonly AudioDeviceViewModel None;

		static AudioDeviceViewModel()
		{
			None = new AudioDeviceViewModel();
		}

		private AudioDeviceViewModel()
		{
			IsInputDevice = true;
			IsOutputDevice = true;
			Device = PortAudioExtensions.NoDevice;
			Name = Device.name;
		}

		public AudioDeviceViewModel(PortAudioSharp.PortAudio.PaDeviceInfo device)
		{
			Device = device;

			var inputChannelCount = device.maxInputChannels;
			var outputChannelCount = device.maxOutputChannels;
			IsInputDevice = inputChannelCount > 0;
			IsOutputDevice = outputChannelCount > 0;

			Name = device.name;
			if (IsInputDevice && IsOutputDevice)
				Name += " (IN: " + inputChannelCount + "  OUT: " + outputChannelCount + ")";
			else if (IsInputDevice)
				Name += " (IN: " + inputChannelCount + ")";
			else if (IsOutputDevice)
				Name += " (OUT: " + outputChannelCount + ")";
		}

		public string Name { get; private set; }

		public bool IsInputDevice { get; private set; }
		public bool IsOutputDevice { get; private set; }

		public PortAudioSharp.PortAudio.PaDeviceInfo Device { get; private set; }

		public override string ToString()
		{
			return Name;
		}
	}
}
