﻿// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using Byond.Standalone.PortAudioStuff;
using Mvvm;
using PortAudioSharp;
using Utils.CtorSubstitution;
using Utils.DialogManagement;

namespace Byond.Standalone.ViewModels
{
	class SettingsViewModel : ViewModelBase
	{
		private readonly IDialogManager _dialogManager;

		[In]
		public AudioSettingsViewModel AudioSettings { get; private set; }

		[In]
		public MidiSettingsViewModel MidiSettings { get; private set; }

		public Action CloseRequest { private get; set; }

		public void ApplySettings()
		{
			var action = string.Empty;
			try
			{
				action = "audio";
				AudioSettings.ApplySettings();

				action = "midi";
				MidiSettings.ApplySettings();

				CloseRequest();
			}
			catch (Exception e)
			{
				_dialogManager
					.CreateMessageDialog(
						"Could not apply " + action + " settings: " + e.Message,
						DialogMode.Ok)
					.Show();
			}
		}
	}
}
