// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

using System;
using Jacobi.Vst.Framework;
using Jacobi.Vst.Framework.Plugin;

namespace Byond.Vst
{
	class Plugin : VstPluginWithInterfaceManagerBase, IVstPluginMidiSource
	{
		public Plugin(
			ByondPluginInfo pluginInfo,
			PluginPersistence pluginPersistence,
			Func<IVstHost, VstMidi> midiFactory,
			Func<IVstHost, VstAudioEngine> audioEngineFactory,
			Func<PluginAudioProcessor> pluginAudioProcessorFactory)
			: base(
				pluginInfo.Name,
				pluginInfo.ProductInfo,
				pluginInfo.Category,
				pluginInfo.Capabilities,
				pluginInfo.InitialDelay,
				pluginInfo.PluginID)
		{
			_pluginPersistence = pluginPersistence;
			_midiFactory = midiFactory;
			_audioEngineFactory = audioEngineFactory;
			_pluginAudioProcessorFactory = pluginAudioProcessorFactory;
		}

		private readonly PluginPersistence _pluginPersistence;
		private readonly Func<IVstHost, VstMidi> _midiFactory;
		private readonly Func<IVstHost, VstAudioEngine> _audioEngineFactory;
		private readonly Func<PluginAudioProcessor> _pluginAudioProcessorFactory;

		private PluginAudioProcessor _pluginAudioProcessor;
		private IVstMidiProcessor _vstMidi;

		protected override IVstPluginAudioProcessor CreateAudioProcessor(IVstPluginAudioProcessor instance)
		{
			return _pluginAudioProcessor;
		}

		protected override IVstMidiProcessor CreateMidiProcessor(IVstMidiProcessor instance)
		{
			return _vstMidi;
		}

		public override void Open(IVstHost host)
		{
			_audioEngineFactory(host);
			_vstMidi = _midiFactory(host);
			_pluginAudioProcessor = _pluginAudioProcessorFactory();
			
			base.Open(host);
		}

		/// <summary>
		/// Always returns <b>this</b>.
		/// </summary>
		/// <param name="instance">A reference to the default instance or null.</param>
		/// <returns>Returns the default instance <b>this</b>.</returns>
		protected override IVstPluginMidiSource CreateMidiSource(IVstPluginMidiSource instance)
		{
			return this;
		}

		protected override IVstPluginPersistence CreatePersistence(IVstPluginPersistence instance)
		{
			return _pluginPersistence;
		}

		public int ChannelCount
		{
			get
			{
				var midiProcessor = Host.GetInstance<IVstMidiProcessor>();
				return midiProcessor != null ? midiProcessor.ChannelCount : 0;
			}
		}
	}
}