// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.IO;
using Byond.Messages;
using Ionic.Zlib;
using Jacobi.Vst.Core;
using Jacobi.Vst.Framework;
using Byond.Logging;
using Byond.Persistence;
using Utils;
using Utils.Messaging;

namespace Byond.Vst
{
	class PluginPersistence : IVstPluginPersistence
	{
		public PluginPersistence(
			ByondPluginInfo pluginInfo,
			PersistenceManager persistenceManager,
			ILogger logger)
		{
			_persistenceManager = persistenceManager;
			_logger = logger;
			_pluginID = pluginInfo.PluginID;
		}

		private readonly int _pluginID;
		private readonly PersistenceManager _persistenceManager;
		private readonly ILogger _logger;

		private string _programToBeRestored;

		public event MessageEventHandler SendMessage;

		public void ReadPrograms(Stream stream, VstProgramCollection programs)
		{
			try
			{
				var buffer = new byte[stream.Length];
				stream.Read(buffer, 0, (int)stream.Length);
				var json = DeflateStream.UncompressString(buffer);

				_programToBeRestored = json;
			}
			catch (Exception e)
			{
				_logger.Log("Could not restore: " + e.Message);
			}
		}

		public void WritePrograms(Stream stream, VstProgramCollection programs)
		{
			SendMessage(new SaveAllEnsemblesRequest());

			if (!_persistenceManager.ShouldPersistEnsemble)
				return;

			var json = _persistenceManager.PersistEnsembleToJson();
			var buffer = DeflateStream.CompressString(json);

			stream.Write(buffer, 0, buffer.Length);
		}

		public bool CanLoadChunk(VstPatchChunkInfo chunkInfo)
		{
			return chunkInfo.PluginID == _pluginID;
		}

		public void HandleMessage(InitializationStage2Message message)
		{
			if (!string.IsNullOrEmpty(_programToBeRestored))
				_persistenceManager.RestoreEnsemble(_programToBeRestored);
			else
				SendMessage(new PluginPersistenceDoneMessage());

			_programToBeRestored = null;
		}
	}
}