﻿using System;
using System.Threading;
using System.Windows;
using System.Windows.Threading;
using Byond.Messages;
using Byond.Shell;
using Utils;
using Utils.Messaging;

namespace Byond.Vst
{
	class StartupHelper
	{
		private readonly ByondControl _byondControl;
		private readonly WinFormsEditor _winFormsEditor;

		private Window _hackWindow;

		public StartupHelper(ByondControl byondControl, WinFormsEditor winFormsEditor)
		{
			_winFormsEditor = winFormsEditor;

			_byondControl = byondControl;
			_byondControl.Loaded += ByondControlOnLoaded;
		}

		public event MessageEventHandler SendMessage;

		public void Start()
		{
			_byondControl.Dispatcher.Invoke(() =>
				{
					// TODO: Eigentlich brauchen wir den Kram nur, wenn es auch RestoreInfor gibt.
					_hackWindow = new Window
						{
							ShowInTaskbar = false,
							Content = _byondControl,
							AllowsTransparency = true,
							WindowStyle = WindowStyle.None,
							WindowState = WindowState.Minimized
						};
					_hackWindow.Show();

					// TODO: Explizitere Initialisierungs-Workflow machen.
					// Z.B. wegen: AudioEngine wird gestartet, obwohl noch kein UManager da ist.
					SendMessage(new InitializationStage1Message());
				});
		}

		// IMP: Das hier wird benötigt, damit nach Persist überhaupt etwas funktioniert.
		// Es muss also der Editor geöffnet werden; das ist schlecht (siehe TODO Liste)
		private void ByondControlOnLoaded(object sender, RoutedEventArgs routedEventArgs)
		{
			_byondControl.Loaded -= ByondControlOnLoaded;
			_byondControl.Dispatcher.BeginInvoke(() =>
				SendMessage(new InitializationStage2Message()));
		}

		public void HandleMessage(EnsembleRunningMessage message)
		{
			CloseHackWindow();
		}

		public void HandleMessage(PluginPersistenceDoneMessage message)
		{
			CloseHackWindow();
		}

		private void CloseHackWindow()
		{
			if (_hackWindow == null)
			{
				_winFormsEditor.SetContent(_byondControl);
				return;
			}

			_byondControl.Dispatcher.Invoke(() =>
				{
					_hackWindow.Content = null;
					_hackWindow.Close();
					_hackWindow = null;
					_winFormsEditor.SetContent(_byondControl);
				});
		}
	}
}
