// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using Byond.Recording;
using Jacobi.Vst.Core;
using Jacobi.Vst.Framework;
using Byond.AudioMidi;
using Byond.HostObjects;

namespace Byond.Vst
{
	sealed class VstAudioEngine : AudioEngine, IAudio
	{
		private readonly IVstHostSequencer _sequencer;

		private VstAudioBuffer[] _inChannels;
		private VstAudioBuffer[] _outChannels;
		private double[][] _recorderChannels;

		public VstAudioEngine(IVstHost host)
		{
			_sequencer = host.GetInstance<IVstHostSequencer>();
		}

		private double _songTempo;
		public override double SongTempo { get { return _songTempo; } }

		private double _sampleRate;
		public override double SampleRate { get { return _sampleRate; } }

		private double _samplePosition;
		public override double SamplePosition { get { return _samplePosition; } }

		private bool _isTransportPlaying;
		public override bool IsTransportPlaying { get { return _isTransportPlaying; } }

		private int _sampleCountInCycle;
		protected override int SampleCountInCycle { get { return _sampleCountInCycle; } }

		public void PrepareCycle(int sampleCount)
		{
			var timeInfo = _sequencer.GetTime(0);
			_songTempo = timeInfo.Tempo;
			_sampleRate = timeInfo.SampleRate;
			_samplePosition = timeInfo.SamplePosition;
			_isTransportPlaying = timeInfo.Flags.HasFlag(VstTimeInfoFlags.TransportPlaying);
			_sampleCountInCycle = sampleCount;
		}

		//protected override void OnVoicesProcessing()
		//{
		//    // NEIN! Das ist ein groes Problem, wenn die Anwendung als Effekt luft.
		//    // In diesem Fall kann es sein, dass Input und Output referenziell fgleich sind.
		//    // Somit wrde der Output berschrieben.

		//    //// Mute everything (important for some hosts)
		//    //foreach (var outChannel in _outChannels)
		//    //    outChannel[CurrentCycle] = 0;
		//}

		protected override void OnVoicesProcessed()
		{
		}

		protected override double[] GetOutputChannelSamples(int channel)
		{
			return _recorderChannels[channel];
		}

		public void SetBuffers(
			VstAudioBuffer[] inChannels,
			VstAudioBuffer[] outChannels)
		{
			_inChannels = inChannels;
			_outChannels = outChannels;
			_recorderChannels = new double[outChannels.Length][];
		}

		public double GetChannelValue(int channel)
		{
			return _inChannels.Length > channel ? _inChannels[channel][CurrentCycle] : 0;
		}

		public void SetChannelValue(int channel, double value)
		{
			if (_outChannels.Length > channel)
				_outChannels[channel][CurrentCycle] = (float)value;
			//if (_recorderChannels.Length > channel)
			//    _recorderChannels[channel][CurrentCycle] = value;
		}
	}
}