// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Diagnostics;

namespace Byond.AudioMidi
{
	public sealed class DiagnosisManager
	{
		private readonly Stopwatch _stopwatch = new Stopwatch();

		public event Action VoicesCountChanged;

		public double CpuLoad { get; private set; }

		private int _voicesCount;
		public int VoicesCount
		{
			get { return _voicesCount; }
			set
			{
				_voicesCount = value;
				if (VoicesCountChanged != null)
					VoicesCountChanged();
			}
		}

		public void StartCpuMeasure()
		{
			_stopwatch.Restart();
		}

		public void StopCpuMeasure(int cycles, double sampleRate)
		{
			_stopwatch.Stop();
			var secondsPerCycle = _stopwatch.Elapsed.TotalSeconds / cycles;
			var maxCycleTimeInSeconds = 1 / sampleRate;
			CpuLoad = secondsPerCycle / maxCycleTimeInSeconds;
		}
	}
}