﻿// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using Byond.HostObjects;
using Byond.Logging;
using Utils.Documentation;

namespace Byond.Builtin
{
	[Doc]
	public class SampleBuffer : Patch
	{
		public SampleBuffer(ILogger logger, Patch parent = null, string memberName = null, bool registerPatch = false)
			: base(parent, memberName, registerPatch)
		{
			InternalBuffer = SampleBufferInternal.Empty;
			_logger = logger;
		}

		private readonly ILogger _logger;

		private SampleBufferInternal _loadingSampleBuffer = SampleBufferInternal.Empty;

		[Doc]
		public long Length { get { return InternalBuffer.Length; } }

		[Doc]
		public int Channels { get { return InternalBuffer.Channels; } }

		internal SampleBufferInternal InternalBuffer { get; private set; }

		private string _fileName;
		public string FileName
		{
			get { return _fileName; }
			set
			{
				if (value == _fileName)
					return;
				_fileName = value;
				ChangeSample();
			}
		}

		private readonly CallbackController _loadedController = new CallbackController();
		[Doc]
		public void Loaded(Action del)
		{
			_loadedController.Register(del);
		}

		[Doc]
		public Sampler CreateSampler()
		{
			return new Sampler(this);
		}

		private void ChangeSample()
		{
			if (_loadingSampleBuffer != null)
				_loadingSampleBuffer.Dispose();

			// Zuweisung erfolgt in Closure
			_loadingSampleBuffer = new SampleBufferInternal(
				_fileName, 
				that =>
					{
						InternalBuffer.Dispose();
						InternalBuffer = that;
						_loadedController.Raise();
					},
				message => _logger.Log(message));
		}
	}
}