﻿// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using System.Linq;

namespace Byond
{
	public class CodeBehindBindingTarget
	{
		internal CodeBehindBindingTarget(IEnumerable<FieldAndPropWrapper> targets)
		{
			Targets = targets;
			var actionBindings = targets
				.Where(it => it.GetValueMethod != null && it.MemberType == typeof(Action))
				.ToList();

			if (actionBindings.Any())
			{
				_invoke = () =>
					{
						foreach (var it in actionBindings)
							((Action)it.GetValueMethod(0))();
					};
				
				return;
			}

			_getValueMethod = targets.Select(it => it.GetValueMethod).FirstOrDefault();
			CanGetValue = _getValueMethod != null;

			_setValueMethods = targets.Select(it => it.SetValueMethod).ToList();
			CanSetValue = _setValueMethods.Any();
		}

		private readonly IEnumerable<Action<object, int?>> _setValueMethods;
		private readonly Func<int?, object> _getValueMethod;
		private readonly Action _invoke;

		private object _lastValue;

		internal IEnumerable<FieldAndPropWrapper> Targets { get; private set; }

		public bool CanGetValue { get; private set; }
		public bool CanSetValue { get; private set; }
		public bool IsAction { get { return _invoke != null; } }

		public object GetValue(int? index)
		{
			return (CanGetValue || IsAction) ? _getValueMethod(index) : null;
		}

		public void SetValue(object value, int? index)
		{
			if (_lastValue == value)
				return;
			_lastValue = value;
			foreach (var it in _setValueMethods)
				it(value, index);
		}

		public void Invoke()
		{
			_invoke();
		}
	}
}