﻿// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System.IO;

namespace Byond
{
	public class Ensemble
	{
		internal Ensemble(string fileName, IEnsembleUnitOfWork uow)
		{
			_uow = uow;
			FileName = fileName;
		}

		private readonly IEnsembleUnitOfWork _uow;

		private string _patchCode = string.Empty;
		public string PatchCode
		{
			get { return _patchCode; }
			set
			{
				_patchCode = value;
				if (ChangeTrackingEnabled)
					_uow.MarkDirty(this);
			}
		}

		private string _uiCode = string.Empty;
		public string UiCode
		{
			get { return _uiCode; }
			set
			{
				_uiCode = value;
				if (ChangeTrackingEnabled)
					_uow.MarkDirty(this);
			}
		}

		public bool IsDirty
		{
			get { return _uow.IsDirty(this); }
		}

		public bool IsNew { get { return string.IsNullOrEmpty(FileName); } }

		public string FileName { get; set; }
		public string Name { get { return Path.GetFileNameWithoutExtension(FileName); } }

		public bool ChangeTrackingEnabled { get; set; }

		public override string ToString()
		{
			return Name + " (file: " + FileName + ")";
		}
	}
}
