﻿// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using System.Linq;

namespace Byond
{
	class EnsembleUnitOfWork : IEnsembleUnitOfWork
	{
		private readonly List<Ensemble> _dirtyEnsembles = new List<Ensemble>();

		// TODO: Messaging
		public event Action Changed;

		public IEnumerable<Ensemble> DirtyEnsembles { get { return _dirtyEnsembles.ToList(); } }

		public bool IsDirty(Ensemble ensemble)
		{
			return _dirtyEnsembles.Contains(ensemble);
		}

		public void MarkDirty(Ensemble ensemble)
		{
			if (IsDirty(ensemble))
				return;
			_dirtyEnsembles.Add(ensemble);
			OnChanged();
		}

		public void MarkClean(Ensemble ensemble)
		{
			_dirtyEnsembles.Remove(ensemble);
			OnChanged();
		}

		private void OnChanged()
		{
			if (Changed != null)
				Changed();
		}
	}
}