// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using Utils.Documentation;

namespace Byond.HostObjects
{
	[Doc]
	public sealed class Position
	{
		[Doc]
		public Position(int bar, int note96)
		{
			TotalNote96 = bar * 96 + note96;
			IsPositive = TotalNote96 > 0;

			// Normalize
			Bar = TotalNote96 / 96;
			Note96 = TotalNote96 % 96;
		}

		[Doc]
		public Position(double barAndNote96)
			: this((decimal)barAndNote96)
		{
		}

		[Doc]
		public Position(decimal barAndNote96)
			: this(
			(int)(decimal.Floor(barAndNote96)),
			(int)((barAndNote96 - decimal.Floor(barAndNote96)) * 100m))
		{
		}

		[Doc]
		public int Bar { get; private set; }

		[Doc]
		public int Note96 { get; private set; }

		[Doc]
		public int TotalNote96 { get; private set; }

		[Doc]
		public bool IsPositive { get; private set; }

		[Doc]
		public Position Add(Position duration)
		{
			return new Position(Bar + duration.Bar, Note96 + duration.Note96);
		}

		[Doc]
		public Position AddBars(int bar)
		{
			return new Position(Bar + bar, Note96);
		}

		[Doc]
		public Position AddNotes(int note96)
		{
			return new Position(Bar, Note96 + note96);
		}

		#region Equals

		// override object.Equals
		public override bool Equals(object obj)
		{
			if (obj == null || GetType() != obj.GetType())
				return false;

			return TotalNote96 == ((Position)obj).TotalNote96;
		}

		// override object.GetHashCode
		public override int GetHashCode()
		{
			return TotalNote96.GetHashCode();
		}

		#endregion

		#region == and !=, <= and >=, < and >

		[Doc]
		public static bool operator ==(Position a, Position b)
		{
			if (ReferenceEquals(a, b))
				return true;
			if (((object)a == null) || ((object)b == null))
				return false;

			// Return true if the fields match:
			return a.TotalNote96 == b.TotalNote96;
		}

		[Doc]
		public static bool operator !=(Position a, Position b)
		{
			return !(a == b);
		}

		[Doc]
		public static bool operator <=(Position a, Position b)
		{
			if (((object)a == null) || ((object)b == null))
				return false;
			return a.TotalNote96 <= b.TotalNote96;
		}

		[Doc]
		public static bool operator >=(Position a, Position b)
		{
			if (((object)a == null) || ((object)b == null))
				return false;
			return a.TotalNote96 >= b.TotalNote96;
		}
		
		[Doc]
		public static bool operator <(Position a, Position b)
		{
			if (((object)a == null) || ((object)b == null))
				return false;
			return a.TotalNote96 < b.TotalNote96;
		}

		[Doc]
		public static bool operator >(Position a, Position b)
		{
			if (((object)a == null) || ((object)b == null))
				return false;
			return a.TotalNote96 > b.TotalNote96;
		}

		#endregion



		#region implicit conversion

		[Doc]
		public static implicit operator Position(double value)
		{
			return new Position(value);
		}

		[Doc]
		public static implicit operator Position(decimal value)
		{
			return new Position(value);
		}

		#endregion

		public override string ToString()
		{
			return Bar + ":" + Note96;
		}
	}
}
