// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using Utils.Documentation;

namespace Byond.HostObjects
{
	[Doc]
	public sealed class Voice : CallbackControl
	{
		internal Voice(
			VoiceManager voiceManager,
			double sampleRate, 
			double hz, 
			Action<CallbackControl> del, 
			bool stopWhenParentStops)
		{
			_voiceManager = voiceManager;
			_sampleRate = sampleRate;
			Hz = hz;
			StopWhenParentStops = stopWhenParentStops;
			_del = del;
			Children = new List<Voice>();
			
			_voiceManager.AddVoice(this);
		}

		// TODO
		//public Voice(Position cycleDuration, Action<VoiceControl> del)
		//    : this(cycleDuration.TotalNote96 * Patch.SongTempo, del)
		//{
		//}

		private readonly VoiceManager _voiceManager;
		private readonly Action<CallbackControl> _del;

		// IMP: Wenn sich die Samplerate ndert: Mitteilen!
		private readonly double _sampleRate;

		private bool _processEveryCycle;
		private double _maxCount;
		private double _currentCount;

		private double _hz;
		[Doc]
		public double Hz
		{
			get { return _hz; }
			set
			{
				if (value <= 0)
					throw new Exception("Hz value of voice was 0.");
				_hz = value > _sampleRate ? _sampleRate : value;
				_maxCount = _sampleRate / _hz;

				_processEveryCycle = (_hz == _sampleRate);
			}
		}

		internal bool StopWhenParentStops { get; private set; }

		internal List<Voice> Children { get; private set; }

		public override void Stop()
		{
			_voiceManager.RemoveVoice(this);
		}

		internal void Process()
		{
			if (_processEveryCycle)
			{
				_del(this);
				return;
			}

			_currentCount += 1;
			var diff = _currentCount - _maxCount;
			if (diff > 0)
			{
				_currentCount = diff;
				_del(this);
			}
		}
	}
}