// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Diagnostics;
using System.Threading;
using System.Windows.Threading;

namespace Byond.Logging
{
	public class Logger : ILogger, IDisposable
	{
		public Logger()
		{
			ThreadPool.QueueUserWorkItem(o =>
				{
					while (!_isDisposed)
					{
						_logCounterForThisSecond = 0;
						Thread.Sleep(TimeSpan.FromSeconds(1));
					}
				});

			AttachDebugger();
		}

		private bool _isDisposed;
		private string _lastLogMessage = string.Empty;
		private int _logCounterForThisSecond;

		// IMP: Weak machen!
		public event Action<string> NewLogMessage;

		public void Log(string message, bool suppressEqualMessages = false)
		{
			if ((message == _lastLogMessage || _logCounterForThisSecond >= 1000) && suppressEqualMessages)
				return;

			_lastLogMessage = message;
			_logCounterForThisSecond++;

			if (NewLogMessage != null)
				NewLogMessage(message);
		}

		public void Dispose()
		{
			_isDisposed = true;
		}

		[Conditional("DEBUG")]
		private void AttachDebugger()
		{
			var debugListener = new DebugListener();
			debugListener.Writing += message => Log("DEBUG:  " + message);
		}
	}
}