// Copyright (c) 2009, Tom Lokovic
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright notice,
//       this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

using System;
using Utils.Documentation;

namespace Byond.Midi
{
	/// <summary>
	/// Control change message.
	/// </summary>
	[Doc]
	public class ControlChangeMessage : ChannelMessage
	{
		/// <summary>
		/// Construts a Control Change message.
		/// </summary>
		/// <param name="channel">Channel, 0..15, 10 reserved for percussion.</param>
		/// <param name="control">Control, 0..119</param>
		/// <param name="value">Value, 0..127.</param>
		[Doc]
		public ControlChangeMessage(Channel channel, byte control, byte value, int midiDeviceNumber)
			: base(channel, midiDeviceNumber)
		{
			if (value < 0 || value > 127)
				throw new ArgumentOutOfRangeException("control");
			Control = control;
			Value = value;
		}

		/// <summary>
		/// The control for this message.
		/// </summary>
		[Doc]
		public byte Control { get; private set; }

		/// <summary>
		/// Value, 0..127.
		/// </summary>
		[Doc]
		public byte Value { get; private set; }

		public override byte[] ToByte()
		{
			return new[] { (byte)(0xB0 | (byte)Channel), Control, Value};
		}
	}
}