﻿// Copyright (c) 2009, Tom Lokovic
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright notice,
//       this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

namespace Byond.Midi
{
	public static class ScalePatterns
	{
		/// <summary>
		/// Pattern for Major scales.
		/// </summary>
		public static readonly ScalePattern Major =
			new ScalePattern("Major", new[] { 0, 2, 4, 5, 7, 9, 11 });

		/// <summary>
		/// Pattern for Natural Minor scales.
		/// </summary>
		public static readonly ScalePattern NaturalMinor =
			new ScalePattern("Natural Minor", new[] { 0, 2, 3, 5, 7, 8, 10 });

		/// <summary>
		/// Pattern for Harmonic Minor scales.
		/// </summary>
		public static readonly ScalePattern HarmonicMinor =
			new ScalePattern("Harmonic Minor", new[] { 0, 2, 3, 5, 7, 8, 11 });

		/// <summary>
		/// Pattern for Melodic Minor scale as it ascends.
		/// </summary>
		public static readonly ScalePattern MelodicMinorAscending =
			new ScalePattern("Melodic Minor (ascending)",
				new[] { 0, 2, 3, 5, 7, 9, 11 });

		/// <summary>
		/// Pattern for Melodic Minor scale as it descends.
		/// </summary>
		public static ScalePattern MelodicMinorDescending =
			new ScalePattern("Melodic Minor (descending)",
				new[] { 0, 2, 3, 5, 7, 8, 10 });

		/// <summary>
		/// Pattern for Chromatic scales.
		/// </summary>
		public static readonly ScalePattern Chromatic =
			new ScalePattern("Chromatic",
				new[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 });
	}
}
