// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System.Diagnostics;
using System.Windows;
using System.Windows.Input;
using Byond.UIRuntime;
using Utils.Documentation;

namespace Byond.PatchUiControls
{
	[Doc]
	public class Button : ButtonBase
	{
		#region Click DP

		public static readonly DependencyProperty ClickProperty =
			DependencyProperty.Register(
				"Click", typeof(string), typeof(Button),
				new PropertyMetadata(default(string)));

		[Doc]
		public string Click
		{
			get { return (string)GetValue(ClickProperty); }
			set { SetValue(ClickProperty, value); }
		}

		#endregion

		protected override void OnPreviewMouseLeftButtonDown(MouseButtonEventArgs e)
		{
			Debug.WriteLine("DOWN");
			base.OnPreviewMouseLeftButtonDown(e);

			CanBindFrom = false;
			CaptureMouse();
			
			var wasChecked = IsChecked;
			IsChecked = true;
			if (!wasChecked)
				WasChangedByUser = true;
		}

		protected override void OnPreviewMouseLeftButtonUp(MouseButtonEventArgs e)
		{
			base.OnPreviewMouseLeftButtonUp(e);

			CanBindFrom = true;
			ReleaseMouseCapture();
			
			var wasChecked = IsChecked;
			IsChecked = false;
			if (wasChecked)
				WasChangedByUser = true;
		}
	}
}