// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System.Windows;
using System.Windows.Input;
using Byond.UIRuntime;
using Utils.Documentation;

namespace Byond.PatchUiControls
{
	[Doc]
	public class MultiButton : UiControl
	{
		static MultiButton()
		{
			DefaultStyleKeyProperty.OverrideMetadata(
				typeof(MultiButton),
				new FrameworkPropertyMetadata(typeof(MultiButton)));
		}

		private int _count = 1;
		[Doc]
		public int Count
		{
			get { return _count; }
			set
			{
				if (value < 1)
					value = 1;
				if (value > 6)
					value = 6;
				_count = value;
				// nochmal update machen
				Value = _value;
			}
		}

		private int _value;
		[Doc(Summary = "TODO: Starts with 0")]
		public int Value
		{
			get { return _value; }
			set
			{
				if (value >= Count)
					value = Count - 1;
				_value = value;
				OnPropertyChanged("Value");
				WasChangedByUser = true;
			}
		}

		private void CycleNextEnum()
		{
			Value = (_value + 1 >= Count) 
				? 0
				: _value + 1;
		}

		protected override void OnPreviewMouseLeftButtonDown(MouseButtonEventArgs e)
		{
			CanBindFrom = false;
			CaptureMouse();
			CycleNextEnum();
			e.Handled = true;
		}

		protected override void OnPreviewMouseLeftButtonUp(MouseButtonEventArgs e)
		{
			CanBindFrom = true;
			ReleaseMouseCapture();
			e.Handled = true;
		}
	}
}
