// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Windows;
using Byond.UIRuntime;
using Utils;
using Utils.Documentation;

namespace Byond.PatchUiControls
{
	[Doc]
	public class Poti : RangeSliderBase
	{
		static Poti()
		{
			DefaultStyleKeyProperty.OverrideMetadata(
				typeof(Poti),
				new FrameworkPropertyMetadata(typeof(Poti)));
		}

		public Poti()
		{
			IsSimpleSlider = true;
		}

		private FrameworkElement _innerFill = new FrameworkElement();
		private FrameworkElement _outerFill = new FrameworkElement();

		public override void OnApplyTemplate()
		{
			base.OnApplyTemplate();
			_innerFill = GetTemplatePart<FrameworkElement>("PART_InnerFill");
			_outerFill = GetTemplatePart<FrameworkElement>("PART_OuterFill");
			_outerFill.SizeChanged += (s, e) => UpdateRadius();
		}

		protected override void OnValuesChanged()
		{
			UpdateRadius();
		}

		private void UpdateRadius()
		{
			var width = MathHelper.Map(Min, 0, Max, _outerFill.ActualWidth, Value);
			var height = MathHelper.Map(Min, 0, Max, _outerFill.ActualHeight, Value);

			_innerFill.Width = Math.Max(0, width);
			_innerFill.Height = Math.Max(0, height);
			//var minVisualHeight = IsSimpleSlider ? 0 : 0; // doch beides 0...
			//_innerFill.Height = height <= 0 ? minVisualHeight : height;
		}
	}
}