// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System.IO;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Win32;
using Byond.UIRuntime;
using Utils.Documentation;

namespace Byond.PatchUiControls
{
	[Doc]
	public class SamplePicker : UiControl
	{
		static SamplePicker()
		{
			DefaultStyleKeyProperty.OverrideMetadata(
				typeof(SamplePicker),
				new FrameworkPropertyMetadata(typeof(SamplePicker)));
		}

		public SamplePicker()
		{
			CanBindFrom = false;
			_dialog = new OpenFileDialog();
		}

		private readonly OpenFileDialog _dialog;

		#region FileName DP

		[Doc]
		public string FileName
		{
			get { return (string)GetValue(FileNameProperty); }
			set { SetValue(FileNameProperty, value); }
		}

		// Using a DependencyProperty as the backing store for FileName.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty FileNameProperty =
			DependencyProperty.Register(
				"FileName",
				typeof(string),
				typeof(SamplePicker),
				new UIPropertyMetadata(
					string.Empty, 
					(s, e) =>
						{
							var me = (SamplePicker)s;
							me.OnPropertyChanged("DisplayFileName");
						}));

		#endregion

		public string DisplayFileName
		{
			get { return string.IsNullOrEmpty(FileName) ? string.Empty : Path.GetFileName(FileName); }
		}

		public override void OnApplyTemplate()
		{
			base.OnApplyTemplate();
			var border = Template.FindName("PART_Border", this) as Border;
			border.PreviewMouseLeftButtonDown += (s, e) => ShowSamplePickDialog();
		}

		private void ShowSamplePickDialog()
		{
			if (!_dialog.ShowDialog().Value)
				return;
			FileName = _dialog.FileName;
			WasChangedByUser = true;
		}
	}
}