// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Windows;
using Utils.Documentation;

namespace Byond.PatchUiControls
{
	[Doc]
	public class TabControl : System.Windows.Controls.TabControl
	{
		static TabControl()
		{
			DefaultStyleKeyProperty.OverrideMetadata(
				typeof(TabControl),
				new FrameworkPropertyMetadata(typeof(TabControl)));
		}

		// No 'jumping' of layout desired, so give it the max size
		protected override Size MeasureOverride(Size constraint)
		{
			var maxHeight = 0d;
			var maxWidth = 0d;

			foreach (TabItem item in Items)
			{
				var content = item.Content as FrameworkElement;
				if (content == null)
					continue;

				if (!content.IsMeasureValid)
					content.Measure(constraint);

				maxHeight = Math.Max(content.DesiredSize.Height, maxHeight);
				maxWidth = Math.Max(content.DesiredSize.Width, maxWidth);
			}

			return new Size(
				maxWidth + Padding.Left + Padding.Right,
				maxHeight + Padding.Top + Padding.Bottom);
		}
	}
}