﻿// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using NAudio.Wave;

namespace Byond.Recording
{
	class MergedChannelsWaveFileWriter : IDisposable
	{
		private readonly int _channelCount;
		private readonly WaveFileWriter _writer;

		public MergedChannelsWaveFileWriter(
			int sampleRate, int channelCount, string fileNameWithoutExtension)
		{
			_channelCount = channelCount;
			_writer = new WaveFileWriter(
				fileNameWithoutExtension + ".wav",
				WaveFormat.CreateIeeeFloatWaveFormat(sampleRate, channelCount));
		}

		public void Write(double[][] channelValues)
		{
			if (channelValues.Length > _channelCount)
				throw new Exception("Channel cound mismatch.");

			// TODO: Exception: Cannot access a closed file.
			var sampleCount = channelValues[0].Length;
			for (var i = 0; i < sampleCount; i++)
				for (var j = 0; j < _channelCount; j++)
					_writer.WriteSample((float)channelValues[j][i]);
		}

		public void Dispose()
		{
			_writer.Dispose();
		}
	}
}