﻿// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using System.Linq;
using NAudio.Wave;

namespace Byond.Recording
{
	class MultiChannelWaveFileWriter : IDisposable
	{
		private readonly List<WaveFileWriter> _writers = new List<WaveFileWriter>();

		public MultiChannelWaveFileWriter(
			int sampleRate, int channelCount, string fileNameWithoutExtension)
		{
			for (var i = 0; i < channelCount; i++)
			{
				_writers.Add(
					new WaveFileWriter(
						fileNameWithoutExtension + "-CH" + i.ToString("D2") + ".wav",
						WaveFormat.CreateIeeeFloatWaveFormat(sampleRate, 1)));
			}
		}

		public void Write(double[][] channelValues)
		{
			if (channelValues.Length > _writers.Count)
				throw new Exception("Channel cound mismatch.");

			for (var i = 0; i < channelValues.Length; i++)
			{
				var channel = channelValues[i];
				var writer = _writers[i];
				writer.WriteSamples(Array.ConvertAll(channel, d => (float)d), 0, channel.Length);
				writer.Flush();
			}
		}

		public void Dispose()
		{
			_writers.ForEach(it => it.Dispose());
			_writers.Clear();
		}
	}
}
