﻿// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using Autofac;

namespace Byond
{
	sealed class RuntimeScope
	{
		private readonly ILifetimeScope _lifetimeScope;
		
		private ILifetimeScope _currentScope;
		private ILifetimeScope _futureScope;

		public RuntimeScopeContext CurrentContext { get; private set; }
		public RuntimeScopeContext FutureContext { get; private set; }
		public RuntimeScopeContext MostFutureContext { get; private set; }

		public void HandleMessage(InitializationStage1Message message)
		{
			Reset();
		}

		public void Reset()
		{
			if (_futureScope != null)
				_futureScope.Dispose();
			_futureScope = null;
			FutureContext = null;

			if (_currentScope != null)
				_currentScope.Dispose();
			_currentScope = _lifetimeScope.BeginLifetimeScope();
			CurrentContext = _currentScope.Resolve<RuntimeScopeContext>();
			MostFutureContext = CurrentContext;
		}

		public void PrepareFutureContext()
		{
			if (_futureScope != null)
				_futureScope.Dispose();
			_futureScope = _lifetimeScope.BeginLifetimeScope();
			FutureContext = _futureScope.Resolve<RuntimeScopeContext>();
			MostFutureContext = FutureContext;
		}

		public void ApplyFutureContext()
		{
			if(_currentScope != null)
				_currentScope.Dispose();
			_currentScope = _futureScope;
			_futureScope = null;
			CurrentContext = FutureContext;
			MostFutureContext = CurrentContext;
			FutureContext = null;
		}

		public void DiscardFutureContext()
		{
			_futureScope = null;
			MostFutureContext = CurrentContext;
			FutureContext = null;
		}
	}
}
