﻿// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Windows.Controls;
using Byond.HostObjects;
using Byond.Shell.ViewModels;
using Byond.Shell.Views;
using Byond.UIRuntime;
using Mvvm;
using Utils.CtorSubstitution;

namespace Byond
{
	sealed class RuntimeScopeContext
	{
		private readonly ViewFactory viewFactory;

		private readonly EnsembleManager ensembleManager;
		private readonly Func<Control, string, LiveViewModel> liveViewModelFactory;

		private void Ctor()
		{
			_lazyLiveView = new Lazy<LiveView>(() =>
				(LiveView)viewFactory.GetViewForModel(
					liveViewModelFactory(
						PatchManager.RootPatchControl,
						ensembleManager.ActiveEnsemble.Name)));
		}

		[In]
		public UiManager UiManager { get; private set; }

		[In]
		public PatchManager PatchManager { get; private set; }

		[In]
		public VoiceManager VoiceManager { get; private set; }

		[In]
		public PatchInfo PatchInfo { get; private set; }

		[In]
		public MidiCallbackManager MidiCallbackManager { get; private set; }

		[In]
		public KeyboardCallbackManager KeyboardCallbackManager { get; private set; }

		private Lazy<LiveView> _lazyLiveView;
		public LiveView LiveView { get { return _lazyLiveView.Value; } }
	}
}