// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interop;
using Mvvm;
using Byond.Logging;
using Byond.Messages;
using Byond.Shell.ViewModels;
using Utils.DialogManagement;
using Utils.Messaging;

namespace Byond.Shell
{
	/// <summary>
	/// Interaction logic for VoiceManager.xaml
	/// </summary>
	public partial class ByondControl
	{
		private readonly RuntimeScope _runtimeScope;
		private readonly ViewFactory _viewFactory;
		private readonly EnsembleManager _ensembleManager;

		internal ByondControl(ViewFactory viewFactory, IDialogManager dialogManager, ILogger logger)
		{
			_viewFactory = viewFactory;

			// TODO: Das benutzt noch keiner...
			Dispatcher.UnhandledException += (s, e) =>
				{
					if (!(e.Exception is DisplayMessageException))
						return;

					e.Handled = true;

					var ex = (DisplayMessageException)e.Exception;
					logger.Log("Dispatcher generic exception occurred: " + ex.ToString());
					dialogManager
						.CreateMessageDialog(ex.Message, ex.Caption, DialogMode.Ok)
						.Show();
				};

			InitializeComponent();

			ShowView(typeof(CodeEditorViewModel));
		}

		public event MessageEventHandler SendMessage;

		public void HandleMessage(EnsembleStoppedMessage message)
		{
			ShowView(typeof(CodeEditorViewModel));
		}

		public void HandleMessage(ShowViewRequest message)
		{
			if (message.ViewModelType != null)
				ShowView(message.ViewModelType);
			else
				ShowContent(() => message.View);
		}

		public void HandleMessage(PreparePatchControlRequest message)
		{
			ShowContent(() => _runtimeScope.MostFutureContext.LiveView);
		}

		private void ShowView(Type viewModelType)
		{
			ShowContent(() => _viewFactory.GetViewForModel(viewModelType));
		}

		// Der Func ist wichtig, dass nicht schon der Content erzeugt wird, obwohl
		// das im Dispatcher noch gar nicht ausgefhrt wird
		private void ShowContent(Func<object> getView)
		{
			// TODO: Dispose old content?
			var setContent = new Action(() => Content = getView());

			if (Dispatcher.CheckAccess())
				setContent();
			else
				Dispatcher.Invoke(setContent);
		}

		protected override void OnKeyDown(KeyEventArgs e)
		{
			if (e.Key == Key.S
				&& (Keyboard.PrimaryDevice.IsKeyDown(Key.LeftCtrl)
					|| Keyboard.PrimaryDevice.IsKeyDown(Key.RightCtrl)))
			{
				SendMessage(new SaveAllEnsemblesRequest());
				e.Handled = true;
			}
			else if (e.Key == Key.F5)
			{
				if (_ensembleManager.ActiveEnsemble != null)
				{
					SendMessage(new RunActiveEnsembleRequest());
				}
			}

			base.OnPreviewKeyDown(e);
		}
	}
}
