using System;
using System.Drawing;
using System.Linq;
using System.Windows;
using System.Windows.Forms;
using System.Xml;
using ICSharpCode.AvalonEdit.Editing;
using ICSharpCode.AvalonEdit.Highlighting;
using ICSharpCode.AvalonEdit.Highlighting.Xshd;

namespace Byond.Shell.Controls
{
	/// <summary>
	/// Interaction logic for AvalonEditor.xaml
	/// </summary>
	public partial class AvalonEditor
	{
		// HACK: Das mit der RichTextBox ist ein einziger Hack, um den
		// ganzen MFC/WPF Interop-Kram zu umgehen.
		// Das muss irgendwann mal richtig gelst werden.

		static AvalonEditor()
		{
			RegisterHighlightDefinition("XmlDoc2");
			RegisterHighlightDefinition("CSLight");
			RegisterHighlightDefinition("XAML");
		}

		public static bool UseRichTextBox;

		private static void RegisterHighlightDefinition(string docName)
		{
			HighlightingManager.Instance.RegisterHighlighting(
				docName,
				new string[0],
				() =>
				{
					using (var stream = typeof(AvalonEditor).Assembly.GetManifestResourceStream("Byond.Shell.Controls." + docName + ".xshd"))
					{
						var reader = XmlReader.Create(stream);
						return HighlightingLoader.Load(reader, HighlightingManager.Instance);
					}
				});
		}

		public AvalonEditor()
		{
			InitializeComponent();

			if (UseRichTextBox)
			{
				contentRoot.Children.Remove(textEditor);

				//_winFormsEditor = new Scintilla();
				_richTextBox = new RichTextBox
					{
						Dock = DockStyle.Fill,
						BackColor = Color.FromArgb(255, 29, 29, 29),
						ShowSelectionMargin = true,
						BorderStyle = BorderStyle.None,
						AutoSize = true,
						ScrollBars = RichTextBoxScrollBars.Both,
						WordWrap = false,
						Font = new Font("Consolas", 10),
						ForeColor = Color.WhiteSmoke,
						SelectionTabs = Enumerable.Range(0, 20).Select(i => (i + 1) * 25).ToArray()
					};
				winFormsHost.Child = _richTextBox;
				//winFormsHost.Child = _winFormsEditor;
			}
			else
			{
				contentRoot.Children.Remove(richtextBoxPanel);

				textEditor.TextArea.LeftMargins
					.OfType<LineNumberMargin>()
					.ForEach(it => it.Margin = new Thickness(0, 0, 10, 0));
				textEditor.TextArea.TextView.Margin = new Thickness(10, 0, 0, 0);
			}
		}

		private readonly RichTextBox _richTextBox;
		//private readonly Scintilla _winFormsEditor;

		#region SyntaxHighlighting DP

		public IHighlightingDefinition SyntaxHighlighting
		{
			get { return (IHighlightingDefinition)GetValue(SyntaxHighlightingProperty); }
			set { SetValue(SyntaxHighlightingProperty, value); }
		}

		// Using a DependencyProperty as the backing store for SyntaxHighlighting.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty SyntaxHighlightingProperty =
			DependencyProperty.Register("SyntaxHighlighting", typeof(IHighlightingDefinition), typeof(AvalonEditor), new UIPropertyMetadata(null));

		#endregion

		#region Text DP

		public string Text
		{
			get { return (string)GetValue(TextProperty); }
			set { SetValue(TextProperty, value); }
		}

		// Using a DependencyProperty as the backing store for Text.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty TextProperty =
			DependencyProperty.Register(
				"Text",
				typeof(string),
				typeof(AvalonEditor),
				new UIPropertyMetadata(null, (s, e) =>
					{
						// HACK: Beim 1. Mal soll der Text an den Editor weitergegeben werden
						var editor = (AvalonEditor)s;

						// TODO: Geht nur, wenn im Viewmodel das Prop einmal gesetzt wurde und auch OnPropChg aufgerufen wurde
						if (e.OldValue != null)
							return;

						if (UseRichTextBox)
						{
							editor._richTextBox.Text = e.NewValue.ToString();
							editor._richTextBox.TextChanged += editor.richTextBox_TextChanged;
							//editor._winFormsEditor.Text = e.NewValue.ToString();
							//editor._winFormsEditor.TextChanged += editor.richTextBox_TextChanged;
						}
						else
						{
							editor.textEditor.Text = e.NewValue.ToString();
							editor.textEditor.TextChanged += editor.textEditor_TextChanged;
						}
					}));

		#endregion

		private void textEditor_TextChanged(object sender, EventArgs e)
		{
			Text = textEditor.Text;
		}

		private void richTextBox_TextChanged(object sender, EventArgs e)
		{
			Text = _richTextBox.Text;
			//Text = _winFormsEditor.Text;
		}
	}
}
