// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System.Windows;
using System.Windows.Input;
using Byond.EnsembleLibrary;
using Mvvm;
using Utils.Messaging;

namespace Byond.Shell.ViewModels
{
	class EnsembleEditorViewModel : ViewModelBase
	{
		public EnsembleEditorViewModel(Ensemble ensemble)
		{
			_ensemble = ensemble;
			_sourceCode = _ensemble.PatchCode;
			_uiCode = _ensemble.UiCode;

			ShowSourceCode();
		}

		private readonly Ensemble _ensemble;

		public event MessageEventHandler SendMessage;

		private string _sourceCode;
		public string SourceCode
		{
			get { return _sourceCode; }
			set
			{
				_sourceCode = value;
				_ensemble.PatchCode = value;
				OnPropertyChanged("SourceCode");
			}
		}

		public Visibility SourceCodeVisibility { get; private set; }
		public bool IsInSourceMode { get { return SourceCodeVisibility == Visibility.Visible; } }

		public void ShowSourceCode()
		{
			SourceCodeVisibility = Visibility.Visible;
			UiCodeVisibility = Visibility.Collapsed;
			OnModeChanged();
		}

		private string _uiCode;
		public string UiCode
		{
			get { return _uiCode; }
			set
			{
				_uiCode = value;
				_ensemble.UiCode = value;
				OnPropertyChanged("UiCode");
			}
		}

		public Visibility UiCodeVisibility { get; private set; }
		public bool IsInUiMode { get { return UiCodeVisibility == Visibility.Visible; } }

		public void ShowUiCode()
		{
			SourceCodeVisibility = Visibility.Collapsed;
			UiCodeVisibility = Visibility.Visible;
			OnModeChanged();
		}

		public void ShowPatchInPatchBrowser()
		{
			SendMessage(new LocateCurrentPatchRequest());
		}

		private void OnModeChanged()
		{
			OnPropertyChanged("SourceCodeVisibility");
			OnPropertyChanged("UiCodeVisibility");

			OnPropertyChanged("IsInSourceMode");
			OnPropertyChanged("IsInUiMode");
		}
	}
}