// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System.IO;
using System.Windows;
using Microsoft.Win32;
using Byond.EnsembleLibrary;

namespace Byond.Shell.ViewModels
{
	public class EnsembleSaveDialog
	{
		private readonly Library _library;

		public string FileName { get; private set; }

		public SaveResult SaveAsAndAsk()
		{
			var res = MessageBox.Show(
				"Save this new ensemble?",
				"Attention",
				MessageBoxButton.YesNoCancel);
			switch (res)
			{
				case MessageBoxResult.Yes:
					return SaveAs();
				case MessageBoxResult.No:
					return SaveResult.GoOn;
				default:
					return SaveResult.Cancel;
			}
		}

		public SaveResult SaveAs()
		{
			var dialog = new SaveFileDialog
				{
					DefaultExt = "*" + Library.PatchFileExtension,
					Filter = "Byond Patches (*" + Library.PatchFileExtension + ")|*" + Library.PatchFileExtension,
					InitialDirectory = _library.LibraryDirectory
				};
			var res = dialog.ShowDialog();
			FileName = dialog.FileName;
			return res.Value ? SaveResult.Save : SaveResult.Cancel;
		}
	}
}