// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Threading;
using Byond.Logging;
using Utils.Messaging;

namespace Byond.Shell.ViewModels
{
	internal class LogViewModel
	{
		private readonly ILogger _logger;
		private readonly Lazy<Dispatcher> _dispatcher;

		private readonly ObservableCollection<string> _logs = new ObservableCollection<string> { "Welcome!" };
		public IEnumerable<string> Logs { get { return _logs; } }

		public event MessageEventHandler SendMessage;

		private void Ctor()
		{
			_logger.NewLogMessage += Log;
		}

		private void Log(string message)
		{
			// Hier kann beim beenden eine ObjectDisposedException kommen
			try
			{
				_dispatcher.Value.BeginInvoke(() =>
				{
					_logs.Insert(0, message);
					if (_logs.Count > 500)
						_logs.RemoveAt(_logs.Count - 1);
				});
			}
			catch (ObjectDisposedException)
			{
			}
		}

		public void GotoCode()
		{
			SendMessage(new ShowViewRequest(typeof(CodeEditorViewModel)));
		}

		public void ClearLog()
		{
			_logs.Clear();
		}
	}
}