// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using Byond.PatchUiControls;
using Byond.UIRuntime;
using Utils.Messaging;

namespace Byond.Shell.Views
{
	public partial class LiveView : IToggleButtonAnchor
	{
		private readonly RuntimeScope _runtimeScope;

		public LiveView()
		{
			InitializeComponent();
			Loaded += OnLoaded;
		}

		public event MessageEventHandler SendMessage;

		private readonly List<Key> _pressedKeys = new List<Key>();

		private void OnLoaded(object sender, RoutedEventArgs routedEventArgs)
		{
			// Event nicht wieder abhngen. wenn z.b. ein dialog auf war, muss der fokus wieder gesetzt werden.
			Focus();
			Keyboard.Focus(this);
		}

		protected override void OnKeyDown(KeyEventArgs e)
		{
			if (!_pressedKeys.Contains(e.Key))
			{
				_pressedKeys.Add(e.Key);
				_runtimeScope.CurrentContext.KeyboardCallbackManager.OnKeyDown(e);
			}

			//e.Handled = true;
			base.OnKeyDown(e);
		}

		protected override void OnKeyUp(KeyEventArgs e)
		{
			// Key Handling funktioniert nicht richtig (Down-A, Down-B, Up-A: Dann kommt komischerweise wieder Down-B ??
			if (_pressedKeys.Contains(e.Key))
			{
				_pressedKeys.Remove(e.Key);
				_runtimeScope.CurrentContext.KeyboardCallbackManager.OnKeyUp(e);
			}

			//e.Handled = true;
			base.OnKeyUp(e);
		}

		// TODO
		//public void HandleMessage(MenuItemsChangedMessage message)
		//{
		//    _menuItems.Clear();
		//    // Wenn das nicht gemacht wird, wird das DataTemplate im ItemsControl ignoriert.
		//    message.MenuItems.Select(it => new { UiElement = it() }).ToList().ForEach(_menuItems.Add);
		//}

		#region IToggleButtonAnchor

		public void DisableGroup(ToggleButton sender, string groupName)
		{
			if (string.IsNullOrWhiteSpace(sender.GroupName))
				return;

			this.GetVisualDescendants()
				.OfType<ToggleButton>()
				.Where(it => it.GroupName.ToLower() == groupName.ToLower() && it != sender)
				.ForEach(it => it.IsChecked = false);
		}

		#endregion
	}
}
