// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
namespace Byond.UIRuntime
{
	class Smoother
	{
		private const int NumberOfSteps = 100;

		private double _stepSize;
		private int _currentIteration;
		private double _lastDesiredValue;

		public double CurrentValue { get; private set; }

		private bool _isRunning;
		public bool IsRunning
		{
			get { return _isRunning; }
			private set
			{
				_isRunning = value;
				LastStepFlag = !value;
			}
		}

		/// <summary>
		/// Der Smoother-Process hat einen anderen Zyklus; deshalb muss beachtet werden,
		/// dass der letzte Wert von der UI geschrirben wird, auch wenn der Smoother
		/// schon beendet ist.
		/// </summary>
		public bool LastStepFlag { get; set; }

		public void SetDesiredValue(double desiredValue)
		{
			// TODO: Glttungs-Faktor
			if (desiredValue != _lastDesiredValue)
			{
				_stepSize = (desiredValue - CurrentValue) / NumberOfSteps;
				_currentIteration = 0;
				IsRunning = true;
			}
			_lastDesiredValue = desiredValue;
		}

		public void Process()
		{
			// wichtig: Im Zusammenspiel mit dem Binder muss erst
			// NACHDEM der letzte Zyklus gelaufen ist
			// IsRunning auf false gesetzt werden.
			if (_currentIteration > NumberOfSteps)
				IsRunning = false;

			if (!IsRunning)
				return;

			CurrentValue += _stepSize;
			_currentIteration++;
			if (_currentIteration == NumberOfSteps)
			{
				// Rundungsfehler mssen vermieden werden!
				CurrentValue = _lastDesiredValue;
				_currentIteration++;
			}
		}

		public void ForceCurrentValue(double value)
		{
			IsRunning = false;
			CurrentValue = value;
			_lastDesiredValue = value;
		}
	}
}
