// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using Utils.Documentation;

namespace Byond.UIRuntime
{
	public abstract class UiControl : Control, INotifyPropertyChanged
	{
		static UiControl()
		{
			DefaultStyleKeyProperty.OverrideMetadata(
				typeof(UiControl),
				new FrameworkPropertyMetadata(typeof(UiControl)));
		}

		protected UiControl()
		{
			CanBindFrom = true;

			// Default Style
			// (Achtung: Gilt NICHT fr die Pabels, TabControl, GroupBox)
			Padding = new Thickness(5);
			VerticalAlignment = VerticalAlignment.Center;
			HorizontalAlignment = HorizontalAlignment.Center;
		}

		public bool CanBindFrom { get; protected set; }
		public virtual bool IsSmoothEnabled { get { return false; } }

		#region Label DP

		[Doc]
		public string Label
		{
			get { return (string)GetValue(LabelProperty); }
			set { SetValue(LabelProperty, value); }
		}

		// Using a DependencyProperty as the backing store for Label.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty LabelProperty =
			DependencyProperty.Register("Label", typeof(string), typeof(UiControl), new UIPropertyMetadata(string.Empty));

		#endregion

		#region ControlSize DP

		[Doc]
		public ControlSize Size
		{
			get { return (ControlSize)GetValue(SizeProperty); }
			set { SetValue(SizeProperty, value); }
		}

		// Using a DependencyProperty as the backing store for Size.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty SizeProperty =
			DependencyProperty.Register("Size", typeof(ControlSize), typeof(UiControl), new UIPropertyMetadata(ControlSize.M));

		#endregion

		#region Padding DP

		public static readonly DependencyProperty PaddingProperty =
			DependencyProperty.Register("Padding", typeof(Thickness), typeof(UiControl), new PropertyMetadata(default(Thickness)));

		public Thickness Padding
		{
			get { return (Thickness)GetValue(PaddingProperty); }
			set { SetValue(PaddingProperty, value); }
		}

		#endregion

		// Indicates the binding engine that the user changed the control
		internal bool WasChangedByUser { get; set; }

		protected T GetTemplatePart<T>(string partName)
			where T : class, new()
		{
			var templatePart = Template.FindName(partName, this) as T;
			return templatePart ?? new T();
		}

		public event PropertyChangedEventHandler PropertyChanged;
		protected void OnPropertyChanged(string p)
		{
			if (PropertyChanged != null)
				PropertyChanged(this, new PropertyChangedEventArgs(p));
		}
	}
}