// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using System.Windows.Threading;
using Byond.Messages;
using Byond.PatchUiControls;
using Utils.DialogManagement;
using Utils.Messaging;

namespace Byond.UIRuntime
{
	sealed class UiManager
	{
		public UiManager(Dispatcher dispatcher)
		{
			_dispatcher = dispatcher;
		}

		public event MessageEventHandler SendMessage;

		private readonly List<Binder> _properties = new List<Binder>();
		private readonly Dispatcher _dispatcher;
		private readonly IDialogManager _dialogManager;

		/// <summary>
		/// Should be called 1000 times per second
		/// </summary>
		public void RefreshBindings()
		{
			BindOrSmooth(property => property.Bind());
		}

		public void RefreshSmoothings()
		{
			BindOrSmooth(property => property.Smooth());
		}

		public void InvokeClickActions()
		{
			foreach (var it in _properties)
				it.InvokeCLickAction();
		}

		private void BindOrSmooth(Action<Binder> del)
		{
			lock (this)
			{
				if (_dispatcher == null)
					return;
				_dispatcher.BeginInvoke(() =>
				{
					try
					{
						foreach (var it in _properties)
							del(it);
					}
					catch (Exception e)
					{
						SendMessage(new StopEnsembleRequest());
						_dialogManager
							.CreateMessageDialog(e.Message, "Error in UI Binding", DialogMode.Ok)
							.Show();
					}
				});
			}			
		}

		public void RegisterBinding(Binder bindingProperty)
		{
			lock (this)
				_properties.Add(bindingProperty);
		}
	}
}