// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Media;

namespace Byond
{
	public static class VisualTreeExtensions
	{
		public static IEnumerable<DependencyObject> GetLogicalDescendants(this DependencyObject depObject, bool includeMe = false)
		{
			var children = new List<DependencyObject>();
			if (depObject == null)
				return children;
			if (includeMe)
				children.Add(depObject);
			foreach (var it in LogicalTreeHelper.GetChildren(depObject).OfType<DependencyObject>().ToList())
			{
				children.Add(it);
				children.AddRange(it.GetLogicalDescendants());
			}
			return children;
		}

		public static IEnumerable<DependencyObject> GetVisualDescendants(this DependencyObject depObject, bool includeMe = false)
		{
			var children = new List<DependencyObject>();
			if (includeMe)
				children.Add(depObject);
			for (int i = 0; i < VisualTreeHelper.GetChildrenCount(depObject); i++)
			{
				var it = VisualTreeHelper.GetChild(depObject, i);
				children.Add(it);
				children.AddRange(it.GetVisualDescendants());
			}
			return children;
		}

		public static DependencyObject GetParent(this DependencyObject depObject)
		{
			return VisualTreeHelper.GetParent(depObject);
		}

		public static T FindParentOfType<T>(this DependencyObject depObject)
			where T : class
		{
			var parent = depObject;
			while (true)
			{
				parent = LogicalTreeHelper.GetParent(parent);
				if (parent == null)
					return null;
				if (parent is T)
					return parent as T;
			}
		}
	}
}
