// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System.Collections;
using System.Linq;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using Utils;

namespace Mvvm.Bindings
{
	class ListBoxBinding : ViewBinding
	{
		public override void SetupView(ViewFactory viewFactory, FrameworkElement view, object viewModel)
		{
			const string selectedText = "Selected";
			viewModel.GetType().GetProperties()
				.Select(it => new
					{
						ListBox = view.FindName(it.Name).AsExactly<ListBox>(),
						Property = it
					})
				.Where(it => it.ListBox != null)
				.ForEach(it =>
					{
						// TODO: Sehr hnlich ItemsControl
						it.ListBox.ItemTemplate = new DynamicDataTemplate(viewFactory).DataTemplate;
						// TODO: Die ganzen Dinge mit Binding machen anstatt direkt zuweisen (auch an den anderen Stellen)
						var propertyValue = it.Property.GetValue(viewModel, null);
						it.ListBox.ItemsSource = propertyValue is IEnumerable
							? (IEnumerable)propertyValue
							: Enumerable.Empty<object>();
					})
				.Where(it => it.Property.Name.StartsWith(selectedText))
				.Select(it => new
					{
						ListBox = view.FindName(it.Property.Name.Remove(0, selectedText.Length) + "s").AsExactly<ListBox>(),
						it.Property
					})
				.Where(it => it.ListBox != null)
				.ForEach(it => it.ListBox.SetBinding(Selector.SelectedItemProperty, it.Property.Name));
		}
	}
}