// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System.Linq;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using Utils;

namespace Mvvm.Bindings
{
	public class SimplePropertyToViewBinding<T> : ViewBinding
		where T : FrameworkElement
	{
		public SimplePropertyToViewBinding(DependencyProperty propertyToBind)
		{
			_propertyToBind = propertyToBind;
		}

		private readonly DependencyProperty _propertyToBind;

		public override void SetupView(ViewFactory viewFactory, FrameworkElement view, object viewModel)
		{
			viewModel.GetType().GetProperties()
				.Select(it => new
				{
					FrameworkElement = view.FindName(it.Name).AsExactly<T>(),
					Property = it
				})
				.Where(it => it.FrameworkElement != null)
				.Where(it => it.FrameworkElement.GetBindingExpression(_propertyToBind) == null)
				.ForEach(it =>
					{
						if (it.Property.GetSetMethod() != null)
						{
							var binding = new Binding(it.Property.Name)
							{
								UpdateSourceTrigger = UpdateSourceTrigger.PropertyChanged,
								Mode = it.Property.GetGetMethod() != null ? BindingMode.TwoWay : BindingMode.OneWayToSource
							};
							it.FrameworkElement.SetBinding(_propertyToBind, binding);
						}
						else
							it.FrameworkElement.SetBinding(_propertyToBind, it.Property.Name);
					});
		}
	}
}