// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Markup;
using System.Xml;

namespace Mvvm
{
	class DynamicDataTemplate
	{
		public readonly static Dictionary<string, ViewFactory> ViewFactories
			= new Dictionary<string, ViewFactory>();

		public DynamicDataTemplate(ViewFactory viewFactory)
		{
			string key;
			if (ViewFactories.ContainsValue(viewFactory))
				key = ViewFactories.FirstOrDefault(it => it.Value == viewFactory).Key;
			else
			{
				key = Guid.NewGuid().ToString();
				ViewFactories[key] = viewFactory;
			}

			var templateString = @"
				<DataTemplate 
						xmlns='http://schemas.microsoft.com/winfx/2006/xaml/presentation' 
						xmlns:x='http://schemas.microsoft.com/winfx/2006/xaml'
						xmlns:mvvm='clr-namespace:Mvvm;assembly=Mvvm'>
					<mvvm:ItemWrapper ViewFactoryKey='INSERT_KEY' />
				</DataTemplate>";
			templateString = templateString.Replace("INSERT_KEY", key);
			DataTemplate = (DataTemplate)XamlReader.Load(
				XmlReader.Create(new StringReader(templateString)));
		}

		public DataTemplate DataTemplate { get; private set; }
	}
}
