﻿// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using System.Linq;

public static class EnumerableExtensions
{
	public static IEnumerable<T> ForEach<T>(this IEnumerable<T> that, Action<T> del)
	{
		var forEach = that as IList<T> ?? that.ToList();
		foreach (var it in forEach)
		{
			del(it);
		}

		return forEach;
	}

	public static T[] Add<T>(this T[] that, T element)
		where T : class
	{
		var copy = new T[that.Length + 1];
		Array.Copy(that, 0, copy, 0, that.Length);
		copy[copy.Length - 1] = element;
		return copy;
	}

	public static T[] Remove<T>(this IEnumerable<T> that, T element)
		where T : class
	{
		// TODO: Performance?
		return that.Where(it => it != element).ToArray();
	}
}
