// Copyright 2013 Ronald Schlenker, www.technewlogic.de.
// 
// This file is part of Byond.
// 
// GraphIT is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// Foobar is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Drawing;
using System.IO;
using System.Reflection;
using Autofac;
using Byond.Shell;
using Byond.Shell.Controls;
using Byond.Vst.GlobalKeyLogger;
using Jacobi.Vst.Core.Plugin;
using Jacobi.Vst.Framework;
using Jacobi.Vst.Framework.Plugin;
using Byond.AudioMidi;
using Utils;

namespace Byond.Vst
{
	public class PluginStub : StdPluginCommandStub
	{
		private readonly FrameworkBootstrapper _frameworkBootstrapper;

		private readonly WinFormsEditor _editor;
		private readonly ByondControl _byondControl;
		private readonly Plugin _pluginInstance;
		private readonly VstPluginInfo _pluginInfo;
		private readonly StartupHelper _startupHelper;
		//private readonly KeyboardDispatcher _keyboardDispatcher;

		static PluginStub()
		{
			AppDomain.CurrentDomain.AssemblyResolve += CurrentDomain_AssemblyResolve;
		}

		public PluginStub()
		{
			// HACK
			AvalonEditor.UseRichTextBox = true;

			_frameworkBootstrapper = new FrameworkBootstrapper(
				cb =>
				{
					cb.RegisterType<VstAudioEngine>().AsSelf().As<IAudio>().As<IAudioInfo>().SingleInstance();
					cb.RegisterType<VstMidi>().AsSelf().As<IMidi>().SingleInstance();

					cb.RegisterType<WinFormsEditor>().SingleInstance();
					cb.RegisterType<PluginResolveContext>().SingleInstance();
					cb.RegisterType<Plugin>().SingleInstance();
					cb.RegisterType<PluginAudioProcessor>().SingleInstance();

					cb.RegisterType<ByondPluginInfo>();
					cb.RegisterType<PluginPersistence>().SingleInstance();

					cb.RegisterType<KeyboardDispatcher>().SingleInstance();
					cb.RegisterType<StartupHelper>().SingleInstance();
				},
				(menuItemRegistrar, viewFactory) => { });

			var pluginContext = _frameworkBootstrapper.Container.Resolve<PluginResolveContext>();
			_pluginInfo = pluginContext.PluginInfo;
			_pluginInstance = pluginContext.Plugin;
			_editor = pluginContext.Editor;
			_byondControl = pluginContext.ByondControl;
			_startupHelper = pluginContext.StartupHelper;
			//_keyboardDispatcher = pluginContext.KeyboardDispatcher;
		}

		private static Assembly CurrentDomain_AssemblyResolve(object sender, ResolveEventArgs args)
		{
			var folderPath = Path.GetDirectoryName(
				Assembly.GetExecutingAssembly().Location);
			var assemblyPath = Path.Combine(
				folderPath,
				new AssemblyName(args.Name).Name + ".dll");
			if (File.Exists(assemblyPath) == false)
				return null;
			return Assembly.LoadFrom(assemblyPath);
		}

		protected override VstPluginInfo CreatePluginInfo(IVstPlugin plugin)
		{
			return _pluginInfo;
		}

		protected override IVstPlugin CreatePluginInstance()
		{
			return _pluginInstance;
		}

		public override bool EditorGetRect(out Rectangle rect)
		{
			rect = _editor.Bounds;
			return true;
		}

		public override bool EditorOpen(IntPtr hWnd)
		{
			// sptestens hier muss der Content gesetzt werden
			_editor.SetContent(_byondControl);
			_editor.Open(hWnd);
			return true;
		}

		//public override void EditorClose()
		//{
		//    _editor.Close();
		//}

		public override void Open()
		{
			base.Open();
			_startupHelper.Start();
		}

		//public override VstCanDoResult BeginLoadBank(VstPatchChunkInfo chunkInfo)
		//{
		//    Logger.Log("begin Load bank");
		//    return VstCanDoResult.Yes;
		//    return base.BeginLoadBank(chunkInfo);
		//}

		//#region Keyboard stuff

		//public override bool EditorKeyDown(byte ascii, VstVirtualKey virtualKey, VstModifierKeys modifers)
		//{
		//    // In Cubase wird der Input gar nicht an das Plugin weitergeleitet (abgesehen von den Pfeiltasten).
		//    // Diese Methode wird dann auch aufgerufen.

		//    // In AL wird der Input weitergeleitet.
		//    // Diese Methode wird NICHT aufgerufen.

		//    //Logger.Log("ascii: " + ascii + " - Virt: " + virtualKey);

		//    //_keyboardDispatcher.IsEnabled = true;
		//    //return true;

		//    if (Keyboard.FocusedElement == null)
		//        return false;

		//    //if (ascii == 0 && virtualKey != VstVirtualKey.VKEY_NONE)
		//    //{
		//    //    InputManager.Current.ProcessInput(
		//    //        new KeyEventArgs(
		//    //            Keyboard.PrimaryDevice,
		//    //            Keyboard.PrimaryDevice.ActiveSource,
		//    //            0,
		//    //            virtualKey.ToWpfKey())
		//    //            {
		//    //                RoutedEvent = Keyboard.PreviewKeyDownEvent
		//    //            });
		//    //}
		//    //else
		//    //{
		//    ModifierKeys modifier;
		//    switch (modifers)
		//    {
		//        case VstModifierKeys.MODIFIER_SHIFT:
		//            modifier = ModifierKeys.Shift;
		//            break;
		//        case VstModifierKeys.MODIFIER_ALTERNATE:
		//            modifier = ModifierKeys.Alt;
		//            break;
		//        case VstModifierKeys.MODIFIER_COMMAND:
		//            modifier = ModifierKeys.Windows;
		//            break;
		//        case VstModifierKeys.MODIFIER_CONTROL:
		//            modifier = ModifierKeys.Control;
		//            break;
		//        case VstModifierKeys.MODIFIER_ALTERNATE | VstModifierKeys.MODIFIER_CONTROL:
		//            modifier = ModifierKeys.Alt | ModifierKeys.Control;
		//            break;
		//        default:
		//            modifier = ModifierKeys.None;
		//            break;
		//    }

		//    // Funktioniert nicht mehr, da _keyboardDispatcher.LastKeyDownEvent null ist
		//    /*
		//    // HACK: ascii == 0: Sollte eigentlich gar nicht passieren.
		//    // In diesem Fall nehmen wir den State des globalen Keyloggers
		//    var wpfKey = ascii != 0
		//        ? ascii.ToWpfKey()
		//        : _keyboardDispatcher.LastKeyDownEvent.Key;
		//    SendKeys.Send(Keyboard.FocusedElement as UIElement, wpfKey, modifier, true, false);
		//    //}
		//    */
		//    if (ascii != 0)
		//    {
		//        var wpfKey = ascii.ToWpfKey();
		//        SendKeys.Send(Keyboard.FocusedElement as UIElement, wpfKey, modifier, true, false);
		//    }

		//    return true;
		//}

		//public override bool EditorKeyUp(byte ascii, VstVirtualKey virtualKey, VstModifierKeys modifers)
		//{
		//    if (Keyboard.FocusedElement == null)
		//        return false;

		//    // Funktioniert nicht mehr, da _keyboardDispatcher.LastKeyDownEvent null ist
		//    /*
		//    // HACK: ascii == 0: Sollte eigentlich gar nicht passieren.
		//    // In diesem Fall nehmen wir den State des globalen Keyloggers
		//    var wpfKey = ascii != 0
		//        ? ascii.ToWpfKey()
		//        : _keyboardDispatcher.LastKeyUpEvent.Key;
		//    SendKeys.Send(Keyboard.FocusedElement as UIElement, wpfKey, ModifierKeys.None, false, true);
		//    */
		//    if (ascii != 0)
		//    {
		//        var wpfKey = ascii.ToWpfKey();
		//        SendKeys.Send(Keyboard.FocusedElement as UIElement, wpfKey, ModifierKeys.None, false, true);
		//    }

		//    return true;
		//}

		//#endregion
	}
}
